@@ -105,7 +105,7 @@ public class HttpAsyncService implements NHttpServerEventHandler {
     static final String HTTP_EXCHANGE_STATE = "http.nio.http-exchange-state";
 
     private final HttpProcessor httpProcessor;
-    private final ConnectionReuseStrategy connStrategy;
+    private final ConnectionReuseStrategy connectionStrategy;
     private final HttpResponseFactory responseFactory;
     private final HttpAsyncRequestHandlerMapper handlerMapper;
     private final HttpAsyncExpectationVerifier expectationVerifier;
@@ -213,7 +213,7 @@ public class HttpAsyncService implements NHttpServerEventHandler {
             final ExceptionLogger exceptionLogger) {
         super();
         this.httpProcessor = Args.notNull(httpProcessor, "HTTP processor");
-        this.connStrategy = connStrategy != null ? connStrategy :
+        this.connectionStrategy = connStrategy != null ? connStrategy :
                 DefaultConnectionReuseStrategy.INSTANCE;
         this.responseFactory = responseFactory != null ? responseFactory :
                 DefaultHttpResponseFactory.INSTANCE;
@@ -312,9 +312,8 @@ public class HttpAsyncService implements NHttpServerEventHandler {
                 closeHandlers(state);
                 if (ex instanceof RuntimeException) {
                     throw (RuntimeException) ex;
-                } else {
-                    log(ex);
                 }
+                log(ex);
             }
         }
     }
@@ -688,7 +687,7 @@ public class HttpAsyncService implements NHttpServerEventHandler {
     private void completeRequest(
             final Incoming incoming,
             final NHttpServerConnection conn,
-            final State state) throws IOException, HttpException {
+            final State state) throws IOException {
         state.setRequestState(MessageState.READY);
         state.setIncoming(null);
 
@@ -743,7 +742,7 @@ public class HttpAsyncService implements NHttpServerEventHandler {
     private void completeResponse(
             final Outgoing outgoing,
             final NHttpServerConnection conn,
-            final State state) throws IOException, HttpException {
+            final State state) throws IOException {
         final HttpContext context = outgoing.getContext();
         final HttpResponse response = outgoing.getResponse();
         final HttpAsyncResponseProducer responseProducer = outgoing.getProducer();
@@ -755,7 +754,7 @@ public class HttpAsyncService implements NHttpServerEventHandler {
         } finally {
             responseProducer.close();
         }
-        if (!this.connStrategy.keepAlive(response, context)) {
+        if (!this.connectionStrategy.keepAlive(response, context)) {
             conn.close();
         } else {
             conn.requestInput();
@@ -1070,6 +1069,8 @@ public class HttpAsyncService implements NHttpServerEventHandler {
 
     /**
      * Adaptor class to transition from HttpAsyncRequestHandlerResolver to HttpAsyncRequestHandlerMapper.
+     *
+     * @deprecated Do not use.
      */
     @Deprecated
     private static class HttpAsyncRequestHandlerResolverAdapter implements HttpAsyncRequestHandlerMapper {
@@ -1087,4 +1088,64 @@ public class HttpAsyncService implements NHttpServerEventHandler {
 
     }
 
+    /**
+     * Gets the HttpResponseFactory for this service.
+     *
+     * @return  the HttpResponseFactory for this service.
+     * @since 4.4.8
+     */
+    public HttpResponseFactory getResponseFactory() {
+      return responseFactory;
+    }
+
+    /**
+     * Gets the HttpProcessor for this service.
+     *
+     * @return the HttpProcessor for this service.
+     * @since 4.4.9
+     */
+    public HttpProcessor getHttpProcessor() {
+        return httpProcessor;
+    }
+
+    /**
+     * Gets the ConnectionReuseStrategy for this service.
+     *
+     * @return the ConnectionReuseStrategy for this service.
+     * @since 4.4.9
+     */
+    public ConnectionReuseStrategy getConnectionStrategy() {
+        return connectionStrategy;
+    }
+
+    /**
+     * Gets the HttpAsyncRequestHandlerMapper for this service.
+     *
+     * @return the HttpAsyncRequestHandlerMapper for this service.
+     * @since 4.4.9
+     */
+    public HttpAsyncRequestHandlerMapper getHandlerMapper() {
+        return handlerMapper;
+    }
+
+    /**
+     * Gets the HttpAsyncExpectationVerifier for this service.
+     *
+     * @return the HttpAsyncExpectationVerifier for this service.
+     * @since 4.4.9
+     */
+    public HttpAsyncExpectationVerifier getExpectationVerifier() {
+        return expectationVerifier;
+    }
+
+    /**
+     * Gets the ExceptionLogger for this service.
+     *
+     * @return the ExceptionLogger for this service.
+     * @since 4.4.9
+     */
+    public ExceptionLogger getExceptionLogger() {
+        return exceptionLogger;
+    }
+
 }
