@@ -65,8 +65,8 @@ import org.apache.http.util.Asserts;
  *  SSLContext sslContext = SSLContext.getInstance("SSL");
  *  sslContext.init(null, null, null);
  *  SSLIOSession sslsession = new SSLIOSession(
- *      iosession, SSLMode.CLIENT, sslContext, null);
- *  iosession.setAttribute(SSLIOSession.SESSION_KEY, sslsession);
+ *      ioSession, SSLMode.CLIENT, sslContext, null);
+ *  ioSession.setAttribute(SSLIOSession.SESSION_KEY, sslsession);
  * </pre>
  *
  * @since 4.2
@@ -300,7 +300,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
                // Just wrap an empty buffer because there is no data to write.
                result = doWrap(ByteBuffer.allocate(0), outEncryptedBuf);
 
-               if (result.getStatus() != Status.OK) {
+               if (result.getStatus() != Status.OK || result.getHandshakeStatus() == HandshakeStatus.NEED_WRAP) {
                    handshaking = false;
                }
                break;
@@ -358,15 +358,21 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
     private void updateEventMask() {
         // Graceful session termination
+        if (this.status == ACTIVE
+                && (this.endOfStream || this.sslEngine.isInboundDone())) {
+            this.status = CLOSING;
+        }
         if (this.status == CLOSING && !this.outEncrypted.hasData()) {
             this.sslEngine.closeOutbound();
         }
         if (this.status == CLOSING && this.sslEngine.isOutboundDone()
-                && (this.endOfStream || this.sslEngine.isInboundDone())) {
+                && (this.endOfStream || this.sslEngine.isInboundDone())
+                && !this.inPlain.hasData()
+                && this.appBufferStatus != null && !this.appBufferStatus.hasBufferedInput()) {
             this.status = CLOSED;
         }
         // Abnormal session termination
-        if (this.status == ACTIVE && this.endOfStream
+        if (this.status <= CLOSING && this.endOfStream
                 && this.sslEngine.getHandshakeStatus() == HandshakeStatus.NEED_UNWRAP) {
             this.status = CLOSED;
         }
@@ -393,6 +399,10 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
             break;
         }
 
+        if (this.endOfStream && (this.appBufferStatus == null || !this.appBufferStatus.hasBufferedInput())) {
+            newMask = newMask & ~EventMask.READ;
+        }
+
         // Do we have encrypted data ready to be sent?
         if (this.outEncrypted.hasData()) {
             newMask = newMask | EventMask.WRITE;
@@ -437,13 +447,16 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         final ByteBuffer inEncryptedBuf = this.inEncrypted.acquire();
 
         // Perform operation
-        final int ret = this.session.channel().read(inEncryptedBuf);
+        final int bytesRead = this.session.channel().read(inEncryptedBuf);
 
         // Release if empty
         if (inEncryptedBuf.position() == 0) {
             this.inEncrypted.release();
         }
-        return ret;
+        if (bytesRead == -1) {
+            this.endOfStream = true;
+        }
+        return bytesRead;
     }
 
     private boolean decryptData() throws SSLException {
@@ -471,9 +484,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
                     }
                     break;
                 }
-                if (result.getHandshakeStatus() != HandshakeStatus.NOT_HANDSHAKING) {
-                    break;
-                }
             } finally {
                 // Release inEncrypted if empty
                 if (this.inEncrypted.acquire().position() == 0) {
@@ -481,6 +491,9 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
                 }
             }
         }
+        if (this.sslEngine.isInboundDone()) {
+            this.endOfStream = true;
+        }
         return decrypted;
     }
 
@@ -492,10 +505,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
      */
     public synchronized boolean isAppInputReady() throws IOException {
         do {
-            final int bytesRead = receiveEncryptedData();
-            if (bytesRead == -1) {
-                this.endOfStream = true;
-            }
+            receiveEncryptedData();
             doHandshake();
             final HandshakeStatus status = this.sslEngine.getHandshakeStatus();
             if (status == HandshakeStatus.NOT_HANDSHAKING || status == HandshakeStatus.FINISHED) {
@@ -587,13 +597,8 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
                 this.inPlain.release();
             }
             return n;
-        } else {
-            if (this.endOfStream) {
-                return -1;
-            } else {
-                return 0;
-            }
         }
+        return this.endOfStream ? -1 : 0;
     }
 
     @Override
@@ -768,11 +773,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
     @Override
     public Socket getSocket(){
-        if (this.session instanceof SocketAccessor){
-            return ((SocketAccessor) this.session).getSocket();
-        } else {
-            return null;
-        }
+        return this.session instanceof SocketAccessor ? ((SocketAccessor) this.session).getSocket() : null;
     }
 
     private class InternalByteChannel implements ByteChannel {
