@@ -433,7 +433,7 @@ public class TestChunkDecoder {
         try {
             decoder2.read(dst);
             Assert.fail("MessageConstraintException expected");
-        } catch (MessageConstraintException ex) {
+        } catch (final MessageConstraintException ex) {
         }
     }
 
@@ -470,7 +470,7 @@ public class TestChunkDecoder {
         try {
             decoder2.read(dst);
             Assert.fail("MessageConstraintException expected");
-        } catch (MessageConstraintException ex) {
+        } catch (final MessageConstraintException ex) {
         }
     }
 
@@ -506,7 +506,7 @@ public class TestChunkDecoder {
         try {
             decoder2.read(dst);
             Assert.fail("MessageConstraintException expected");
-        } catch (MessageConstraintException ex) {
+        } catch (final MessageConstraintException ex) {
         }
     }
 
@@ -543,7 +543,7 @@ public class TestChunkDecoder {
         try {
             decoder2.read(dst);
             Assert.fail("MessageConstraintException expected");
-        } catch (MessageConstraintException ex) {
+        } catch (final MessageConstraintException ex) {
         }
     }
 
